@@ -43,8 +43,6 @@ class Cluster(resource.Resource):
     project_id = resource.Body('project')
     #: The domain ID of the cluster owner.
     domain_id = resource.Body('domain')
-    #: The ID of the parent cluster (if any).
-    parent_id = resource.Body('parent')
     #: Timestamp of when the cluster was initialized.
     #: *Type: datetime object parsed from ISO 8601 formatted string*
     init_at = resource.Body('init_at')
@@ -76,6 +74,8 @@ class Cluster(resource.Resource):
     node_ids = resource.Body('nodes')
     #: Name of the profile used by the cluster.
     profile_name = resource.Body('profile_name')
+    #: A dictionary with dependency information of the cluster
+    dependents = resource.Body('dependents', type=dict)
 
     def action(self, session, body):
         url = utils.urljoin(self.base_path, self._get_id(self), 'actions')
